@@ -18,9 +18,8 @@ module Statistics.Distribution.ChiSquared (
         , chiSquaredNDF
         ) where
 
-import Data.Typeable (Typeable)
-import Statistics.Constants (m_huge)
-import Statistics.Math (incompleteGamma,logGamma)
+import Data.Typeable         (Typeable)
+import Numeric.SpecFunctions (incompleteGamma,invIncompleteGamma,logGamma)
 
 import qualified Statistics.Distribution as D
 
@@ -58,6 +57,13 @@ instance D.Variance ChiSquared where
     variance (ChiSquared ndf) = fromIntegral (2*ndf)
     {-# INLINE variance #-}
 
+instance D.MaybeMean ChiSquared where
+    maybeMean = Just . D.mean
+
+instance D.MaybeVariance ChiSquared where
+    maybeStdDev   = Just . D.stdDev
+    maybeVariance = Just . D.variance
+
 cumulative :: ChiSquared -> Double -> Double
 cumulative chi x
   | x <= 0    = 0
@@ -77,8 +83,10 @@ density chi x
 {-# INLINE density #-}
 
 quantile :: ChiSquared -> Double -> Double
-quantile d@(ChiSquared ndf) p
-  | p == 0    = -1/0
-  | p == 1    = 1/0
-  | otherwise = D.findRoot d p (fromIntegral ndf) 0 m_huge
+quantile (ChiSquared ndf) p
+  | p == 0         = 0
+  | p == 1         = 1/0
+  | p > 0 && p < 1 = 2 * invIncompleteGamma (fromIntegral ndf / 2) p
+  | otherwise      =
+    error $ "Statistics.Distribution.ChiSquared.quantile: p must be in [0,1] range. Got: "++show p
 {-# INLINE quantile #-}
