@@ -1,7 +1,7 @@
 {-# LANGUAGE DeriveDataTypeable #-}
 -- |
 -- Module    : Statistics.Distribution.Gamma
--- Copyright : (c) 2009 Bryan O'Sullivan
+-- Copyright : (c) 2009, 2011 Bryan O'Sullivan
 -- License   : BSD3
 --
 -- Maintainer  : bos@serpentine.com
@@ -25,9 +25,10 @@ module Statistics.Distribution.Gamma
     ) where
 
 import Data.Typeable (Typeable)
-import Statistics.Constants (m_huge)
-import Statistics.Math (incompleteGamma, logGamma)
-import qualified Statistics.Distribution as D
+import Numeric.MathFunctions.Constants (m_pos_inf, m_NaN)
+import Numeric.SpecFunctions           (incompleteGamma, invIncompleteGamma)
+import Statistics.Distribution.Poisson.Internal as Poisson
+import qualified Statistics.Distribution        as D
 
 -- | The gamma distribution.
 data GammaDistribution = GD {
@@ -35,7 +36,8 @@ data GammaDistribution = GD {
     , gdScale :: {-# UNPACK #-} !Double -- ^ Scale parameter, &#977;.
     } deriving (Eq, Read, Show, Typeable)
 
--- | Create gamma distrivution. Both shape and scale parameters must be positive.
+-- | Create gamma distribution. Both shape and scale parameters must
+-- be positive.
 gammaDistr :: Double            -- ^ Shape parameter. /k/
            -> Double            -- ^ Scale parameter, &#977;.
            -> GammaDistribution
@@ -54,17 +56,30 @@ instance D.ContDistr GammaDistribution where
     quantile   = quantile
 
 instance D.Variance GammaDistribution where
-    variance (GD a l) = a / (l * l)
+    variance (GD a l) = a * l * l
     {-# INLINE variance #-}
 
 instance D.Mean GammaDistribution where
-    mean (GD a l) = a / l
+    mean (GD a l) = a * l
     {-# INLINE mean #-}
 
+instance D.MaybeMean GammaDistribution where
+    maybeMean = Just . D.mean
+
+instance D.MaybeVariance GammaDistribution where
+    maybeStdDev   = Just . D.stdDev
+    maybeVariance = Just . D.variance
+
+
+
 density :: GammaDistribution -> Double -> Double
 density (GD a l) x
-  | x <= 0    = 0
-  | otherwise = x ** (a-1) * exp (-x/l) / (exp (logGamma a) * l ** a)
+  | a < 0 || l <= 0   = m_NaN
+  | x <= 0            = 0
+  | a == 0            = if x == 0 then m_pos_inf else 0
+  | x == 0            = if a < 1 then m_pos_inf else if a > 1 then 0 else 1/l
+  | a < 1             = Poisson.probability (x/l) a * a / x
+  | otherwise         = Poisson.probability (x/l) (a-1) / l
 {-# INLINE density #-}
 
 cumulative :: GammaDistribution -> Double -> Double
@@ -74,8 +89,10 @@ cumulative (GD k l) x
 {-# INLINE cumulative #-}
 
 quantile :: GammaDistribution -> Double -> Double
-quantile d p
-  | p == 0    = -1/0
-  | p == 1    = 1/0
-  | otherwise = D.findRoot d p (gdShape d) 0 m_huge
+quantile (GD k l) p
+  | p == 0         = 0
+  | p == 1         = 1/0
+  | p > 0 && p < 1 = l * invIncompleteGamma k p
+  | otherwise      =
+    error $ "Statistics.Distribution.Gamma.quantile: p must be in [0,1] range. Got: "++show p
 {-# INLINE quantile #-}
